@@ -1,12 +1,16 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 2001-8 The R Development Core Team.
+ *  Copyright (C) 2001-11 The R Core Team.
  *
- *  This program is free software; you can redistribute it and/or modify
+ *  This header file is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
  *  the Free Software Foundation; either version 2.1 of the License, or
  *  (at your option) any later version.
  *
+ *  This file is part of R. R is distributed under the terms of the
+ *  GNU General Public License, either Version 2, June 1991 or Version 3,
+ *  June 2007. See doc/COPYRIGHTS for details of the copyright status of R.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -14,7 +18,7 @@
  *
  *  You should have received a copy of the GNU Lesser General Public License
  *  along with this program; if not, a copy is available at
- *  http://www.r-project.org/Licenses/
+ *  https://www.R-project.org/Licenses/
  */
 
 /* Used by graphics.c, grid and by third-party graphics devices */
@@ -29,8 +33,8 @@ extern "C" {
 /*
  * The current graphics engine (including graphics device) API version
  * MUST be integer
- * 
- * This number should be bumped whenever there are changes to 
+ *
+ * This number should be bumped whenever there are changes to
  * GraphicsEngine.h or GraphicsDevice.h so that add-on packages
  * that compile against these headers (graphics systems such as
  * graphics and grid;  graphics devices such as gtkDevice, RSvgDevice)
@@ -38,31 +42,48 @@ extern "C" {
  *
  * Version 1:  Introduction of the version number.
  * Version 2:  GEDevDesc *dd dropped from GEcontourLines().
- * Version 3:  R_GE_str2col() added to API.
+ * Version 3:  R_GE_str2col() added to API. (r41887)
  * Version 4:  UTF-8 text hooks, useRotatedTextInContour,
  *             add newFrameConfirm() to NewDevDesc.
  *             New API: GEaddDevice[2] GEgetDevice, GEkillDevice,
- *             ndevNumber.
+ *             ndevNumber. (R 2.7.0)
  * Version 5:  Clean up 1.4.0/2.0.0 changes!
  *             Remove newDevStruct from GEDevDesc and NewDevDesc.
  *             Remove asp, dot(), hold(), open() from NewDevDesc.
  *             Move displayList, DLlastElt, savedSnapshot from
  *             NewDevDesc to GEDevDesc.
- *             Add 'ask' to GEDevDesc.
+ *             Add 'ask' to GEDevDesc. (R 2.8.0)
+ * Version 6:  Add dev_Raster() and dev_Cap()  (R 2.11.0)
+ * Version 7:  Change graphics event handling, adding eventEnv and eventHelper()
+ *	       to DevDesc.  (R 2.12.0)
+ * Version 8:  Add dev_Path() (R 2.12.0)
+ * Version 9:  Add dev_HoldFlush(), haveTrans*, haveRaster,
+ *             haveCapture, haveLocator.  (R 2.14.0)
+ * Version 10: For R 3.0.0.  Typedef and use 'rcolor',
+ *             Remove name2col (R_GE_str2col does the job).
+ * Version 11: For R 3.3.0.
+ *             Official support for saving/restoring display lists
+ *             across R sessions (via recordPlot() and replayPlot())
+ *             - added grid DL to snapshots (used to be NULL)
+ *             - added this version number to snapshots (as attribute)
+ *             - added R version number to snapshots (as attribute)
+ *             - added pkgName to graphics system state info (as attribute)
+ * Version 12: For R 3.4.0
+ *             Added canGenIdle, doIdle() and doesIdle() to devices.
  */
 
-#define R_GE_version 5
+#define R_GE_version 12
 
 int R_GE_getVersion(void);
 
 void R_GE_checkVersionOrDie(int version);
 
-/* The graphics engine will only accept locations and dimensions 
+/* The graphics engine will only accept locations and dimensions
  * in native device coordinates, but it provides the following functions
- * for converting between a couple of simple alternative coordinate 
+ * for converting between a couple of simple alternative coordinate
  * systems and device coordinates:
  *    DEVICE = native units of the device
- *    NDC = Normalised device coordinates 
+ *    NDC = Normalised device coordinates
  *    INCHES = inches (!)
  *    CM = centimetres (!!)
  */
@@ -79,6 +100,9 @@ typedef enum {
 typedef enum {
     /* In response to this event, the registered graphics system
      * should allocate and initialise the systemSpecific structure
+     *
+     * Should return R_NilValue on failure so that engine
+     * can tidy up memory allocation
      */
     GE_InitState = 0,
     /* This event gives the registered system a chance to undo
@@ -110,8 +134,8 @@ typedef enum {
     /* Restore the system state that is saved by GE_SaveSnapshotState
      */
     GE_RestoreSnapshotState = 5,
-    /* When replaying the display list, the graphics engine 
-     * checks, after each replayed action, that the action 
+    /* When replaying the display list, the graphics engine
+     * checks, after each replayed action, that the action
      * produced valid output.  This is the graphics system's
      * chance to say that the output is crap (in which case the
      * graphics engine will abort the display list replay).
@@ -119,7 +143,7 @@ typedef enum {
     GE_CheckPlot = 7,
     /* The device wants to scale the current pointsize
      * (for scaling an image)
-     * This is not a nice general solution, but a quick fix for 
+     * This is not a nice general solution, but a quick fix for
      * the Windows device.
      */
     GE_ScalePS = 8
@@ -140,8 +164,8 @@ typedef enum {
   GE_BEVEL_JOIN = 3
 } R_GE_linejoin;
 
-/* 
- * A structure containing graphical parameters 
+/*
+ * A structure containing graphical parameters
  *
  * This is how graphical parameters are passed from graphics systems
  * to the graphics engine AND from the graphics engine to graphics
@@ -159,7 +183,7 @@ typedef struct {
     int col;             /* pen colour (lines, text, borders, ...) */
     int fill;            /* fill colour (for polygons, circles, rects, ...) */
     double gamma;        /* Gamma correction */
-    /* 
+    /*
      * Line characteristics
      */
     double lwd;          /* Line width (roughly number of pixels) */
@@ -178,7 +202,7 @@ typedef struct {
 } R_GE_gcontext;
 
 typedef R_GE_gcontext* pGEcontext;
-    
+
 
 #include <R_ext/GraphicsDevice.h> /* needed for DevDesc */
 
@@ -186,14 +210,14 @@ typedef struct _GEDevDesc GEDevDesc;
 
 typedef SEXP (* GEcallback)(GEevent, GEDevDesc *, SEXP);
 
-typedef struct { 
+typedef struct {
     /* An array of information about each graphics system that
      * has registered with the graphics engine.
      * This is used to store graphics state for each graphics
      * system on each device.
      */
     void *systemSpecific;
-    /* 
+    /*
      * An array of function pointers, one per graphics system that
      * has registered with the graphics engine.
      *
@@ -210,7 +234,7 @@ typedef struct {
 } GESystemDesc;
 
 struct _GEDevDesc {
-    /* 
+    /*
      * Stuff that the devices can see (and modify).
      * All detailed in GraphicsDevice.h
      */
@@ -231,11 +255,11 @@ struct _GEDevDesc {
     Rboolean recordGraphics; /* Should a graphics call be stored
 			      * on the display list?
 			      * Set to FALSE by do_recordGraphics,
-			      * do_dotcallgr, and do_Externalgr 
+			      * do_dotcallgr, and do_Externalgr
 			      * so that nested calls are not
 			      * recorded on the display list
 			      */
-    /* 
+    /*
      * Stuff about the device that only graphics systems see.
      * The graphics engine has no idea what is in here.
      * Used by graphics systems to store system state per device.
@@ -257,6 +281,7 @@ int GEdeviceNumber(pGEDevDesc);
 pGEDevDesc GEgetDevice(int);
 void GEaddDevice(pGEDevDesc);
 void GEaddDevice2(pGEDevDesc, const char *);
+void GEaddDevice2f(pGEDevDesc, const char *, const char *);
 void GEkillDevice(pGEDevDesc);
 pGEDevDesc GEcreateDevDesc(pDevDesc dev);
 
@@ -276,13 +301,13 @@ SEXP GEhandleEvent(GEevent event, pDevDesc dev, SEXP data);
 #define fromDeviceHeight	GEfromDeviceHeight
 #define toDeviceHeight		GEtoDeviceHeight
 
-double fromDeviceX(double value, GEUnit to, pGEDevDesc dd); 
+double fromDeviceX(double value, GEUnit to, pGEDevDesc dd);
 double toDeviceX(double value, GEUnit from, pGEDevDesc dd);
-double fromDeviceY(double value, GEUnit to, pGEDevDesc dd); 
+double fromDeviceY(double value, GEUnit to, pGEDevDesc dd);
 double toDeviceY(double value, GEUnit from, pGEDevDesc dd);
-double fromDeviceWidth(double value, GEUnit to, pGEDevDesc dd); 
+double fromDeviceWidth(double value, GEUnit to, pGEDevDesc dd);
 double toDeviceWidth(double value, GEUnit from, pGEDevDesc dd);
-double fromDeviceHeight(double value, GEUnit to, pGEDevDesc dd); 
+double fromDeviceHeight(double value, GEUnit to, pGEDevDesc dd);
 double toDeviceHeight(double value, GEUnit from, pGEDevDesc dd);
 
 /*-------------------------------------------------------------------
@@ -292,25 +317,25 @@ double toDeviceHeight(double value, GEUnit from, pGEDevDesc dd);
  *  From colors.c, used in par.c, grid/src/gpar.c
  */
 
+typedef unsigned int rcolor;
+
 #define RGBpar			Rf_RGBpar
 #define RGBpar3			Rf_RGBpar3
 #define col2name                Rf_col2name
-#define name2col		Rf_name2col
 
 /* Convert an element of a R colour specification (which might be a
    number or a string) into an internal colour specification. */
-unsigned int RGBpar(SEXP, int);
-unsigned int RGBpar3(SEXP, int, unsigned int);
+rcolor RGBpar(SEXP, int);
+rcolor RGBpar3(SEXP, int, rcolor);
 
 /* Convert an internal colour specification to/from a colour name */
-const char *col2name(unsigned int col); /* used in par.c, grid */
-unsigned int name2col(const char *);    /* used by plotmath.c */
+const char *col2name(rcolor col); /* used in par.c, grid */
 
 /* Convert either a name or a #RRGGBB[AA] string to internal.
    Because people were using it, it also converts "1", "2" ...
    to a colour in the palette, and "0" to transparent white.
 */
-unsigned int R_GE_str2col(const char *s);
+rcolor R_GE_str2col(const char *s);
 
 
 
@@ -364,19 +389,28 @@ SEXP GE_LJOINget(R_GE_linejoin ljoin);
 
 void GESetClip(double x1, double y1, double x2, double y2, pGEDevDesc dd);
 void GENewPage(const pGEcontext gc, pGEDevDesc dd);
-void GELine(double x1, double y1, double x2, double y2, 
+void GELine(double x1, double y1, double x2, double y2,
 	    const pGEcontext gc, pGEDevDesc dd);
-void GEPolyline(int n, double *x, double *y, 
+void GEPolyline(int n, double *x, double *y,
 		const pGEcontext gc, pGEDevDesc dd);
-void GEPolygon(int n, double *x, double *y, 
+void GEPolygon(int n, double *x, double *y,
 	       const pGEcontext gc, pGEDevDesc dd);
-SEXP GEXspline(int n, double *x, double *y, double *s, Rboolean open, 
+SEXP GEXspline(int n, double *x, double *y, double *s, Rboolean open,
 	       Rboolean repEnds, Rboolean draw,
 	       const pGEcontext gc, pGEDevDesc dd);
 void GECircle(double x, double y, double radius,
 	      const pGEcontext gc, pGEDevDesc dd);
 void GERect(double x0, double y0, double x1, double y1,
 	    const pGEcontext gc, pGEDevDesc dd);
+void GEPath(double *x, double *y,
+            int npoly, int *nper,
+            Rboolean winding,
+            const pGEcontext gc, pGEDevDesc dd);
+void GERaster(unsigned int *raster, int w, int h,
+              double x, double y, double width, double height,
+              double angle, Rboolean interpolate,
+              const pGEcontext gc, pGEDevDesc dd);
+SEXP GECap(pGEDevDesc dd);
 void GEText(double x, double y, const char * const str, cetype_t enc,
 	    double xc, double yc, double rot,
 	    const pGEcontext gc, pGEDevDesc dd);
@@ -384,13 +418,16 @@ void GEMode(int mode, pGEDevDesc dd);
 void GESymbol(double x, double y, int pch, double size,
 	      const pGEcontext gc, pGEDevDesc dd);
 void GEPretty(double *lo, double *up, int *ndiv);
-void GEMetricInfo(int c, const pGEcontext gc, 
+void GEMetricInfo(int c, const pGEcontext gc,
 		  double *ascent, double *descent, double *width,
 		  pGEDevDesc dd);
-double GEStrWidth(const char *str, cetype_t enc, 
+double GEStrWidth(const char *str, cetype_t enc,
 		  const pGEcontext gc, pGEDevDesc dd);
 double GEStrHeight(const char *str, cetype_t enc,
 		  const pGEcontext gc, pGEDevDesc dd);
+void GEStrMetric(const char *str, cetype_t enc, const pGEcontext gc,
+                 double *ascent, double *descent, double *width,
+                 pGEDevDesc dd);
 int GEstring_to_pch(SEXP pch);
 
 /*-------------------------------------------------------------------
@@ -401,39 +438,63 @@ int GEstring_to_pch(SEXP pch);
 unsigned int GE_LTYpar(SEXP, int);
 SEXP GE_LTYget(unsigned int);
 
-/* 
- * From plotmath.c 
+/*
+ * Raster operations
  */
-double GEExpressionWidth(SEXP expr, 
+void R_GE_rasterScale(unsigned int *sraster, int sw, int sh,
+                      unsigned int *draster, int dw, int dh);
+void R_GE_rasterInterpolate(unsigned int *sraster, int sw, int sh,
+                            unsigned int *draster, int dw, int dh);
+void R_GE_rasterRotatedSize(int w, int h, double angle,
+                            int *wnew, int *hnew);
+void R_GE_rasterRotatedOffset(int w, int h, double angle, int botleft,
+                              double *xoff, double *yoff);
+void R_GE_rasterResizeForRotation(unsigned int *sraster,
+                                  int w, int h,
+                                  unsigned int *newRaster,
+                                  int wnew, int hnew,
+                                  const pGEcontext gc);
+void R_GE_rasterRotate(unsigned int *sraster, int w, int h, double angle,
+                       unsigned int *draster, const pGEcontext gc,
+                       Rboolean perPixelAlpha);
+
+
+/*
+ * From plotmath.c
+ */
+double GEExpressionWidth(SEXP expr,
 			 const pGEcontext gc, pGEDevDesc dd);
-double GEExpressionHeight(SEXP expr, 
+double GEExpressionHeight(SEXP expr,
 			  const pGEcontext gc, pGEDevDesc dd);
+void GEExpressionMetric(SEXP expr, const pGEcontext gc,
+                        double *ascent, double *descent, double *width,
+                        pGEDevDesc dd);
 void GEMathText(double x, double y, SEXP expr,
-		double xc, double yc, double rot, 
+		double xc, double yc, double rot,
 		const pGEcontext gc, pGEDevDesc dd);
-/* 
+/*
  * (End from plotmath.c)
  */
 
-/* 
+/*
  * From plot3d.c : used in package clines
  */
 SEXP GEcontourLines(double *x, int nx, double *y, int ny,
 		    double *z, double *levels, int nl);
-/* 
+/*
  * (End from plot3d.c)
  */
 
-/* 
+/*
  * From vfonts.c
  */
 double R_GE_VStrWidth(const char *s, cetype_t enc, const pGEcontext gc, pGEDevDesc dd);
 
 double R_GE_VStrHeight(const char *s, cetype_t enc, const pGEcontext gc, pGEDevDesc dd);
-void R_GE_VText(double x, double y, const char * const s, cetype_t enc, 
+void R_GE_VText(double x, double y, const char * const s, cetype_t enc,
 		double x_justify, double y_justify, double rotation,
 		const pGEcontext gc, pGEDevDesc dd);
-/* 
+/*
  * (End from vfonts.c)
  */
 
@@ -455,9 +516,12 @@ void GEonExit(void);
 void GEnullDevice(void);
 
 
-/* From plot.c, used by grid/src/grid.c */
+/* From ../../main/plot.c, used by ../../library/grid/src/grid.c : */
 #define CreateAtVector		Rf_CreateAtVector
 SEXP CreateAtVector(double*, double*, int, Rboolean);
+/* From ../../main/graphics.c, used by ../../library/grDevices/src/axis_scales.c : */
+#define GAxisPars 		Rf_GAxisPars
+void GAxisPars(double *min, double *max, int *n, Rboolean log, int axis);
 
 #ifdef __cplusplus
 }
