@@ -3,13 +3,11 @@
 
 require(datasets)
 require(grDevices); require(graphics)
-oask <- devAskNewPage(dev.interactive(orNone = TRUE))
-
 
 ## (1) The Obligatory Mathematical surface.
 ##     Rotated sinc function.
 
-x <- seq(-10, 10, length = 50)
+x <- seq(-10, 10, length.out = 50)
 y <- x
 rotsinc <- function(x,y)
 {
@@ -51,7 +49,7 @@ z <- rbind(z0, cbind(z0, z, z0), z0)
 x <- c(min(x) - 1e-10, x, max(x) + 1e-10)
 y <- c(min(y) - 1e-10, y, max(y) + 1e-10)
 
-fill <- matrix("green3", nr = nrow(z)-1, nc = ncol(z)-1)
+fill <- matrix("green3", nrow = nrow(z)-1, ncol = ncol(z)-1)
 fill[ , i2 <- c(1,ncol(fill))] <- "gray"
 fill[i1 <- c(1,nrow(fill)) , ] <- "gray"
 
@@ -77,7 +75,8 @@ fcol <- fill
 zi <- volcano[ -1,-1] + volcano[ -1,-61] +
            volcano[-87,-1] + volcano[-87,-61]  ## / 4
 fcol[-i1,-i2] <-
-    terrain.colors(20)[cut(zi, stats::quantile(zi, seq(0,1, len = 21)),
+    terrain.colors(20)[cut(zi,
+                           stats::quantile(zi, seq(0,1, length.out = 21)),
                            include.lowest = TRUE)]
 persp(x, y, 2*z, theta = 110, phi = 40, col = fcol, scale = FALSE,
       ltheta = -120, shade = 0.4, border = NA, box = FALSE)
@@ -85,4 +84,3 @@ persp(x, y, 2*z, theta = 110, phi = 40, col = fcol, scale = FALSE,
 
 ## reset par():
 par(oldpar)
-devAskNewPage(oask)
